#include <gtk/gtk.h>

#include <stdio.h>
#include <string.h>

#include <dbui-defs.h>

#include <dbui-structs.h>

#include <dbui-prototypes.h>

#include "column_functions.h"

#include "function_prototypes.h"

#define USER_ENTRY "Enter your own value"

#define PREV_VALUE   "Reset value"

#define COL_SET    "Set field = another field"

#define SEARCH_STRING "gtk-combo-string-value"

#define SET_FIELD_STRING "set field = field"

static void 
list_selected(GtkList       *list,GtkCombo      *entry_field);

void
reset_function(GtkMenuItem *menuitem,gpointer object);

void
set_equal_field_func(GtkMenuItem *menuitem,gpointer object);

void
order_by_function(GtkMenuItem *menuitem,gpointer object);


/*
int
create_entry_field	(COLUMN *col,GtkTable *container,int row)

INPUTS: COLUMN that you wish to create an entry field for
	GtkTable that you wish to put the entry field in
	INT row that you want to put it in

We create an entry field based on what type of column it is then we create a label based on the name in col then we attach this label to the new entry field. We also attach the col to the new entry field.

We then attach this new entry field to the container. When this entry field has out lived its time the field and everything attached to it should be destroyed. 

*/

int
create_entry_field	(COLUMN	*col,GtkTable *container,int row)
{
    DBUI_COMBO_ENTRY	*combo_entry;
    TABLE	*table;
    GtkWidget	*entry_field;
    GtkWidget	*entry_label;
    GtkWidget	*align;
    GtkWidget	*align2;
    GtkWidget	*option;
    GtkWidget   *menu;
    GtkWidget	*menu_item;
    GList	*entry_list;
    GList	*combo_list = NULL;
    char	buffer[NAME_LENGTH];
    int		buffer_len = 0;
    int		i = 0;

    if(!col)
	{
	return -1;
	}

    switch (col->type)
	{
        case DBUI_BLOB:
                entry_field = gtk_text_new (NULL, NULL);

                gtk_text_set_editable (GTK_TEXT (entry_field), TRUE);

                gtk_widget_set_usize (entry_field, 222 , 65 );

                break;

	case DBUI_TINY:
	case DBUI_SHORT:
	case DBUI_INT:
	case DBUI_LONG:
	case DBUI_LONGLONG:
	case DBUI_FLOAT:
	case DBUI_DOUBLE:
	case DBUI_DATE:
	case DBUI_TIME:
	case DBUI_DATETIME:
	case DBUI_VARCHAR:
	case DBUI_CHAR:
	case DBUI_SET:
	case DBUI_ENUM:
	default:
		combo_entry = create_new_combo_entry();
	
		append_combo_menu_option(combo_entry->popup_menu,"Reset Value",
						combo_entry,
						reset_function);
		if(col->table && ((TABLE *)col->table)->parent )
			{
		
			append_combo_menu_option(combo_entry->popup_menu,
				"Set = to selected field",
				combo_entry,
				set_equal_field_func);
			append_combo_menu_option(combo_entry->popup_menu,
                                "Order By",
                                combo_entry,
                                order_by_function);

			}
		entry_field = combo_entry->hbox;
		break;

	}

    bzero(buffer,NAME_LENGTH);

    buffer[0] = ' ';

    if(col->table)
	{
	/* We want to make sure we leave space for the column name */
	snprintf (buffer + 1,NAME_LENGTH / 2 , "%s.",col->parent_name);
	}

    /* We print buffer first so we can get table_name.column_name */

    buffer_len =strlen(buffer);

    memcpy(buffer + buffer_len,col->name,NAME_LENGTH /2);

    /* we fill out the rest just so it will line up nicely .... */

    for( i = strlen(buffer) ; i < 32 ; i++)
	buffer[i] = ' ';
	
    buffer[i] = '\0';

    entry_label = gtk_label_new(buffer);

    gtk_widget_show(entry_label);

    gtk_label_set_justify(GTK_LABEL(entry_label),GTK_JUSTIFY_LEFT );

    align = gtk_alignment_new( 0.1 , 0.0 , 0.0 , 0.0 );

    align2 = gtk_alignment_new( 0.00 , 0.0 , 0.0 , 0.0 );

    gtk_container_add(GTK_CONTAINER(align) , entry_label );

    gtk_container_add(GTK_CONTAINER(align2) , entry_field);

    gtk_object_set_data (GTK_OBJECT(entry_field), "ALIGN", align );

    gtk_object_set_data (GTK_OBJECT(entry_field), "ALIGN2", align2 );

    gtk_object_set_data (GTK_OBJECT(entry_field), "LABEL",entry_label);


    /* We attach the column to the field entry and setup the destroy function */

    gtk_object_set_data_full(GTK_OBJECT(entry_field), "COLUMN",col,
			(GtkDestroyNotify)delete_column);

    gtk_object_set_data (GTK_OBJECT(container), buffer, entry_field);

    if(col->need_compair_type)
	{
	option = gtk_option_menu_new();

	gtk_widget_set_usize(option,60,20);

	menu = gtk_menu_new ();

	menu_item = gtk_menu_item_new_with_label("=");

	gtk_object_set_data(GTK_OBJECT(menu),"=",menu_item);

	gtk_menu_append (GTK_MENU (menu),menu_item);

	menu_item = gtk_menu_item_new_with_label("Like");

	gtk_object_set_data(GTK_OBJECT(menu),"Like",menu_item);

        gtk_menu_append (GTK_MENU (menu),menu_item);

        menu_item = gtk_menu_item_new_with_label(">");

	gtk_object_set_data(GTK_OBJECT(menu),">",menu_item);
	
        gtk_menu_append (GTK_MENU (menu),menu_item);

        menu_item = gtk_menu_item_new_with_label("<");

	gtk_object_set_data(GTK_OBJECT(menu),"<",menu_item);

        gtk_menu_append (GTK_MENU (menu),menu_item);

	gtk_option_menu_set_menu (GTK_OPTION_MENU(option),menu);

	gtk_object_set_data(GTK_OBJECT(option),"menu",menu);

	gtk_table_attach_defaults( GTK_TABLE(container), option,
		1 , 2 , row , row + 1 );

	gtk_widget_show_all(option);
	}

    gtk_object_set_data(GTK_OBJECT(entry_field),"OPTION_MENU",option);

    /* Attach the label to the left column */
    gtk_table_attach_defaults( GTK_TABLE(container), align,
			0 , 1 , row , row + 1 );

    /* Attach the field to the right column */
    gtk_table_attach_defaults( GTK_TABLE(container), align2,
			2 , 3 , row , row + 1 );

 
    entry_list = gtk_object_get_data (GTK_OBJECT(container) , "ENTRY_LIST" );

    if(! entry_list)
	{
	entry_list = g_list_append(entry_list,entry_field);

	gtk_object_set_data_full ( GTK_OBJECT(container) , "ENTRY_LIST",
				entry_list,(GtkDestroyNotify)g_list_free );

	}
    else
	{
	g_list_append(entry_list,entry_field);
	}
	
return 0;
}

void
order_by_function(GtkMenuItem *menuitem,gpointer object)
{
    DBUI_COMBO_ENTRY *combo_entry = NULL;
    GtkWidget   *hbox;
    COLUMN      *col = NULL;

    combo_entry = (DBUI_COMBO_ENTRY *)object;

    hbox = combo_entry->hbox;
    col = gtk_object_get_data(GTK_OBJECT(hbox),"COLUMN");

    if(col )
        {
	if(col->order_by)
	  {
	  col->order_by = 0;
	  mark_popup_menu(GTK_WIDGET(menuitem),FALSE);
	  }
	else
	  {
	  col->order_by = 1;
          mark_popup_menu(GTK_WIDGET(menuitem),TRUE);

	  }
	}
}

void
reset_function(GtkMenuItem *menuitem,gpointer object)
{
    DBUI_COMBO_ENTRY *combo_entry = NULL;
    GtkWidget	*hbox;
    GList	*nth = NULL;
    GList	*g_l = NULL;
    COLUMN	*col = NULL;
    COLUMN	*col2 = NULL;
    TABLE	*table = NULL;

    if(!object)
	return;  

    combo_entry = (DBUI_COMBO_ENTRY *)object; 

    hbox = combo_entry->hbox;
    col = gtk_object_get_data(GTK_OBJECT(hbox),"COLUMN");    

    if(col && col->table)
	{
	table = col->table;
	}

    if(table && table->result_set )
	{

	nth = g_list_nth(table->result_set,table->result_index - 1);	
	if(nth)
		g_l = nth->data;
	while(g_l)
	  {
	  if(g_l->data)
		{
		col2 = (COLUMN *)g_l->data;
		if(strcmp(col2->name,col->name) == 0)
		  {
		  col->need_quote = 1;
		  break;
		  }
		}
	  g_l = g_list_next(g_l);
	  }
	if(g_l)
		set_entry_value(hbox,g_l->data);			
	}
}

void
set_equal_field_func(GtkMenuItem *menuitem,gpointer object)
{
    DBUI_COMBO_ENTRY *combo_entry = NULL;
    GtkWidget   *hbox;
    GtkWidget	*main_window;
    COLUMN	*col;
    CLIST_REQUEST *cl_req;

    if(!object)
        return;

    combo_entry = (DBUI_COMBO_ENTRY *)object;
        
    hbox = combo_entry->hbox;

    if(!hbox)
	return;

    col = gtk_object_get_data(GTK_OBJECT(hbox),"COLUMN");

    if(col->equal_col)
	{
	delete_column(col->equal_col);
	col->equal_col = NULL;
	mark_popup_menu((GtkWidget*)menuitem,FALSE);
	return;
	}

    main_window = gtk_widget_get_toplevel(GTK_WIDGET(hbox));

    if(!main_window || !col)
	return;

    cl_req = gtk_object_get_data(GTK_OBJECT(main_window),"CLIST_REQUEST");

    cl_req->type = SET_EQ_COL;
    cl_req->requester = combo_entry;
    cl_req->spare1 = menuitem;


}
